@@ -4,9 +4,7 @@
 **
 **  MODULE
 **
-**      $RCSfile: bibtex.c,v $
-**      $Revision: 3.71 $
-**      $Date: 1996/08/18 20:37:06 $
+**      file: bibtex.c
 **
 **  DESCRIPTION
 **
@@ -105,6 +103,7 @@
 #ifdef KPATHSEA
 #include <kpathsea/config.h>
 #include <kpathsea/progname.h>
+#include <kpathsea/variable.h>
 #endif
 
 #include "sysdep.h"
@@ -233,6 +232,9 @@ int                     main (int argc, char **argv)
 BEGIN
     extern Integer8_T       history;
     int			    exit_status;
+#ifdef KPATHSEA
+    char *mpl;
+#endif
 
     /*-
     **------------------------------------------------------------------------
@@ -265,20 +267,28 @@ BEGIN
     number_of_command_line_args = argc;
     command_line_arg_strings = (char **) argv;
 
-#ifdef KPATHSEA
-    kpse_set_program_name(argv[0], PROGNAME);
-#endif
-
     history = SPOTLESS;
     parse_cmd_line (argc, argv);
 
+#ifdef KPATHSEA
+    kpse_set_program_name(argv[0], PROGNAME);
+    mpl = kpse_var_value("max_print_line");
+    if (mpl)
+    BEGIN
+        max_print_line = atoi(mpl);
+        free(mpl);
+    END
+    else
+        max_print_line = 79;  /* default */
+#endif
+
     set_array_sizes ();
     report_search_paths ();
 
     initialize ();
 
     if (log_file != NULL) {
-        FPRINTF (log_file, "%s\n", BANNER);
+        FPRINTF (log_file, "%s-x%s (%s)\n", BANNER, EXT_VERSION, TL_VERSION);
         FPRINTF (log_file, "Implementation:  %s\n", IMPLEMENTATION);
         FPRINTF (log_file, "Release version: %s\n", VERSION);
 #ifdef UTF_8
