@@ -2,7 +2,7 @@
 ** CMapManager.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,9 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#if defined(MIKTEX)
+#include <config.h>
+#endif
 #include <array>
 #include <sstream>
 #include "CMap.hpp"
@@ -119,7 +122,7 @@ const CMap* CMapManager::findCompatibleBaseFontMap (const PhysicalFont *font, co
 
 
 	// try to find a compatible encoding CMap
-	const bool is_unicode_map = bool(dynamic_cast<const UnicodeCMap*>(cmap));
+	const bool is_unicode_map = cmap->mapsToUnicode();
 	const string ro = cmap->getROString();
 	for (const CharMapIDToEncName &enc : encodings) {
 		for (const CharMapID &id : charmapIDs) {
