@@ -2,7 +2,7 @@
 ** FileSystem.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -112,8 +112,8 @@ bool FileSystem::remove (const string &fname) {
  *  @return true on success */
 bool FileSystem::copy (const string &src, const string &dest, bool remove_src) {
 #if defined(MIKTEX_WINDOWS)
-        ifstream ifs(EXPATH_(src), ios::in | ios::binary);
-        ofstream ofs(EXPATH_(dest), ios::out | ios::binary);
+	ifstream ifs(EXPATH_(src), ios::in | ios::binary);
+	ofstream ofs(EXPATH_(dest), ios::out | ios::binary);
 #else
 	ifstream ifs(src, ios::in|ios::binary);
 	ofstream ofs(dest, ios::out|ios::binary);
@@ -167,6 +167,14 @@ string FileSystem::ensureForwardSlashes (string path) {
 }
 
 
+string FileSystem::ensureSystemSlashes (string path) {
+#ifdef _WIN32
+	std::replace(path.begin(), path.end(), '/', PATHSEP);
+#endif
+	return path;
+}
+
+
 /** Returns the absolute path of the current working directory. */
 string FileSystem::getcwd () {
 	char buf[1024];
@@ -247,12 +255,13 @@ const char* FileSystem::userdir () {
 }
 
 
-/** Returns the path of the temporary folder. */
-string FileSystem::tmpdir () {
+/** Returns the path of the temporary folder.
+ *  @param[in] inpace if true, don't create a uniquely named subfolder */
+string FileSystem::tmpdir (bool inplace) {
 	if (_tmpdir.path().empty()) {
 		string basedir;
 		if (!TMPDIR.empty())
-			basedir = TMPDIR;
+			basedir = ensureForwardSlashes(TMPDIR);
 		else {
 #ifdef _WIN32
 #if defined(MIKTEX)
@@ -272,10 +281,14 @@ string FileSystem::tmpdir () {
 			else
 				basedir = "/tmp";
 #endif
-			if (basedir.back() == '/')
-				basedir.pop_back();
 		}
-		_tmpdir = TemporaryDirectory(basedir, PROGRAM_NAME);
+		if (basedir.length() > 2 && string(basedir.end()-2, basedir.end()) == "//") {
+			inplace = true;
+			basedir.pop_back();
+		}
+		if (basedir.front() != '/' && basedir.back() == '/')
+			basedir.pop_back();
+		_tmpdir = TemporaryDirectory(basedir, PROGRAM_NAME, inplace);
 	}
 	return _tmpdir.path();
 }
@@ -394,7 +407,7 @@ bool FileSystem::isDirectory (const string &fname) {
 bool FileSystem::isFile (const string &fname) {
 	if (const char *cfname = fname.c_str()) {
 #if defined(MIKTEX)
-                return MiKTeX::Core::File::Exists(MiKTeX::Util::PathName(fname));
+		return MiKTeX::Core::File::Exists(MiKTeX::Util::PathName(fname));
 #else
 #ifdef _WIN32
 		ifstream ifs(cfname);
@@ -458,26 +471,34 @@ int FileSystem::collect (const std::string &dirname, vector<string> &entries) {
 }
 
 
-/** Creates a temporary directory in a given folder.
+/** Creates a temporary directory in a given folder or treats the given folder as temporary directory.
  *  @param[in] folder folder path in which the directory is to be created
- *  @param[in] prefix initial string of the directory name */
-FileSystem::TemporaryDirectory::TemporaryDirectory (const std::string &folder, string prefix) {
-	using namespace std::chrono;
-	auto now = system_clock::now().time_since_epoch();
-	auto now_ms = duration_cast<milliseconds>(now).count();
-	auto hash = XXH64HashFunction(to_string(now_ms)).digestValue();
-	if (!prefix.empty() && prefix.back() != '-')
-		prefix += "-";
-	for (int i=0; i < 10 && _path.empty(); i++) {
-		hash++;
-		stringstream oss;
-		oss << folder << '/' << prefix << hex << hash;
-		if (exists(oss.str()))
-			continue;
-		if (s_mkdir(oss.str()))
-			_path = oss.str() + "/";
-		else
-			break;
+ *  @param[in] prefix initial string of the directory name
+ *  @param[in] inplace if true, 'folder' is treated as temporary directory and no subfolder is created */
+FileSystem::TemporaryDirectory::TemporaryDirectory (const std::string &folder, string prefix, bool inplace) {
+	if (inplace) {
+		_path = folder;
+		if (!_path.empty() && _path.back() != '/')
+			_path.push_back('/');
+	}
+	else {
+		using namespace std::chrono;
+		auto now = system_clock::now().time_since_epoch();
+		auto now_ms = duration_cast<milliseconds>(now).count();
+		auto hash = XXH64HashFunction(to_string(now_ms)).digestValue();
+		if (!prefix.empty() && prefix.back() != '-')
+			prefix.push_back('-');
+		for (int i = 0; i < 10 && _path.empty(); i++) {
+			hash++;
+			stringstream oss;
+			oss << folder << '/' << prefix << hex << hash;
+			if (exists(oss.str()))
+				continue;
+			if (s_mkdir(oss.str()))
+				_path = oss.str() + "/";
+			else
+				break;
+		}
 	}
 }
 
