@@ -1,167 +1,200 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
 <TS version="2.1" language="fr_BE">
+<context>
+    <name>GuidelineEditDialog</name>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="14"/>
+        <source>Edit guideline...</source>
+        <translation>Editer le guide...</translation>
+    </message>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="22"/>
+        <source>Page:</source>
+        <translation>Page :</translation>
+    </message>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="36"/>
+        <source>Position:</source>
+        <translation>Position :</translation>
+    </message>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="53"/>
+        <source>cm</source>
+        <translation>cm</translation>
+    </message>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="58"/>
+        <source>in</source>
+        <translation>in (pouces)</translation>
+    </message>
+    <message>
+        <location filename="../src/GuidelineEditDialog.ui" line="63"/>
+        <source>bp</source>
+        <translation>bp (point)</translation>
+    </message>
+</context>
 <context>
     <name>PaperSize</name>
     <message>
         <location filename="../src/PaperSizes.cpp" line="24"/>
         <source>Beamer 16:10</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 16:10</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="25"/>
         <source>Beamer 16:9</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 16:9</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="26"/>
         <source>Beamer 14:9</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 14:9</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="27"/>
         <source>Beamer 1.41:1</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 1.41:1</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="28"/>
         <source>Beamer 5:4</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 5:4</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="29"/>
         <source>Beamer 4:3</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 4:3</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="30"/>
         <source>Beamer 3:2</source>
-        <translation type="unfinished"></translation>
+        <translation>Beamer 3:2</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="33"/>
         <source>Screen</source>
-        <translation type="unfinished"></translation>
+        <translation>Écran</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="36"/>
         <source>DIN 4A0</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN 4A0</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="37"/>
         <source>DIN 2A0</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN 2A0</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="38"/>
         <source>DIN A0</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A0</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="39"/>
         <source>DIN A1</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A1</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="40"/>
         <source>DIN A2</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A2</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="41"/>
         <source>DIN A3</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A3</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="42"/>
         <source>DIN A4</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A4</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="43"/>
         <source>DIN A5</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A5</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="44"/>
         <source>DIN A6</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A6</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="45"/>
         <source>DIN A7</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A7</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="46"/>
         <source>DIN A8</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A8</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="47"/>
         <source>DIN A9</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A9</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="48"/>
         <source>DIN A10</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN A10</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="49"/>
         <source>DIN B0</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN B0</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="50"/>
         <source>DIN B1</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN B1</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="51"/>
         <source>DIN B2</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN B2</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="52"/>
         <source>DIN B3</source>
-        <translation type="unfinished"></translation>
+        <translation>BIN B3</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="53"/>
         <source>DIN B4</source>
-        <translation type="unfinished"></translation>
+        <translation>DIN B4</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="54"/>
         <source>DIN B5</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B5</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="55"/>
         <source>DIN B6</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B6</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="56"/>
         <source>DIN B7</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B7</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="57"/>
         <source>DIN B8</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B8</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="58"/>
         <source>DIN B9</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B9</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="59"/>
         <source>DIN B10</source>
-        <translation type="unfinished"></translation>
+        <translation type="unfinished">DIN B10</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="60"/>
@@ -636,7 +669,7 @@
     <message>
         <location filename="../src/PaperSizes.cpp" line="155"/>
         <source>Half Letter</source>
-        <translation type="unfinished"></translation>
+        <translation>Demi Letter</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="156"/>
@@ -707,24 +740,24 @@
         <location filename="../src/PaperSizes.cpp" line="201"/>
         <source>mm</source>
         <extracomment>Length unit: millimeters</extracomment>
-        <translation type="unfinished"></translation>
+        <translation>mm</translation>
     </message>
     <message>
         <location filename="../src/PaperSizes.cpp" line="207"/>
         <source>in</source>
         <extracomment>Length unit: inches</extracomment>
-        <translation type="unfinished"></translation>
+        <translation>in (pouce)</translation>
     </message>
 </context>
 <context>
     <name>QtPDF</name>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="1269"/>
+        <location filename="../src/PDFDocumentView.cpp" line="33"/>
         <source>[language name]</source>
         <translation>Française</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="1270"/>
+        <location filename="../src/PDFDocumentView.cpp" line="34"/>
         <source>[translator&apos;s name/email]</source>
         <translation>Alain Delmotte/leliseron@yahoo.fr</translation>
     </message>
@@ -737,32 +770,32 @@
         <translation>rendu de la page</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2019"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2029"/>
         <source>Unlock PDF</source>
         <translation>Déverrouiller le PDF</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2019"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2029"/>
         <source>Please enter the password to unlock the PDF</source>
         <translation>Veuillez entrer le mot de passe pour déverrouiller le PDF</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2035"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2045"/>
         <source>Incorrect password</source>
         <translation>Mot de passe incorrect</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2035"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2045"/>
         <source>The password you entered was incorrect.</source>
         <translation>Le mot de passe que vous avez entré est incorrect.</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2041"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2051"/>
         <source>This document is locked. You need a password to open it.</source>
         <translation>Ce document est verrouillé. Vous avez besoin d&apos;un mot de passe pour l&apos;ouvrir.</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2042"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2052"/>
         <source>Unlock</source>
         <translation>Déverrouiller</translation>
     </message>
@@ -780,313 +813,313 @@
         <translation>L&apos;extraction de texte n&apos;est pas permise pour ce document.</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2587"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2597"/>
         <source>Goto page %1</source>
         <translation>Aller à la page %1</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2599"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2609"/>
         <source>Execute `%1`</source>
         <translation>Exécuter `%1´</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2845"/>
-        <location filename="../src/PDFDocumentView.cpp" line="2862"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2855"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2872"/>
         <source>Table of Contents</source>
         <translation>Table des matières</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="2954"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3161"/>
+        <location filename="../src/PDFDocumentView.cpp" line="2964"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3171"/>
         <source>Meta Data</source>
         <translation>Méta donnée</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3010"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3171"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3020"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3181"/>
         <source>Processing</source>
         <translation>Traitement</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3042"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3178"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3052"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3188"/>
         <source>Other</source>
         <translation>Autre</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3067"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3077"/>
         <source>B</source>
         <extracomment>File size: bytes</extracomment>
         <translation>o</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3069"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3079"/>
         <source>kB</source>
         <extracomment>File size: kilobytes</extracomment>
         <translation>ko</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3071"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3081"/>
         <source>MB</source>
         <extracomment>File size: megabytes</extracomment>
         <translation>Mo</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3073"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3083"/>
         <source>GB</source>
         <extracomment>File size: gigabytes</extracomment>
         <translation>Go</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3075"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3085"/>
         <source>TB</source>
         <extracomment>File size: terabytes</extracomment>
         <translation>To</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3105"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3115"/>
         <source>Yes</source>
         <translation>Oui</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3108"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3118"/>
         <source>No</source>
         <translation>Non</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3111"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3148"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3121"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3158"/>
         <source>Unknown</source>
         <translation>Inconnu</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3163"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3173"/>
         <source>Document</source>
         <translation>Document</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3164"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3174"/>
         <source>Title:</source>
         <translation>Titre :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3165"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3175"/>
         <source>Author:</source>
         <translation>Auteur :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3166"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3176"/>
         <source>Subject:</source>
         <translation>Sujet :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3167"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3177"/>
         <source>Keywords:</source>
         <translation>Mots clé :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3168"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3178"/>
         <source>Page size:</source>
         <translation>Dimension de page :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3169"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3179"/>
         <source>File size:</source>
         <translation>Taille du fichier :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3172"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3182"/>
         <source>Creator:</source>
         <translation>Créateur :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3173"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3183"/>
         <source>Producer:</source>
         <translation>Producteur :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3174"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3184"/>
         <source>Creation date:</source>
         <translation>Date de création :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3175"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3185"/>
         <source>Modification date:</source>
         <translation>Date de modification :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3176"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3186"/>
         <source>Trapped:</source>
         <translation>Capturé :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3187"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3281"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3197"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3291"/>
         <source>Fonts</source>
         <translation>Polices</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3237"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3247"/>
         <source>Type 0</source>
         <translation>Type 0</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3240"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3250"/>
         <source>Type 1</source>
         <translation>Type 1</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3243"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3253"/>
         <source>Type 1 (multiple master)</source>
         <translation>Type 1 (attributs multiples)</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3246"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3256"/>
         <source>Type 3</source>
         <translation>Type 3</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3249"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3259"/>
         <source>TrueType</source>
         <translation>TrueType</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3252"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3262"/>
         <source>yes</source>
         <translation>oui</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3252"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3262"/>
         <source>no</source>
         <translation>non</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3255"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3265"/>
         <source>[embedded]</source>
         <translation>[enchâssé]</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3258"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3268"/>
         <source>[builtin]</source>
         <translation>[inclus]</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3282"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3292"/>
         <source>Name</source>
         <translation>Nom</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3282"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3292"/>
         <source>Type</source>
         <translation>Type</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3282"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3292"/>
         <source>Subset</source>
         <translation>Sous-ensemble</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3282"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3292"/>
         <source>Source</source>
         <translation>Source</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3290"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3385"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3300"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3395"/>
         <source>Permissions</source>
         <translation>Permissions</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3339"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3348"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3357"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3364"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3369"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3349"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3358"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3367"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3374"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3379"/>
         <source>Allowed</source>
         <translation>Permis</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3341"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3351"/>
         <source>Low resolution only</source>
         <translation>Basse résolution seulement</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3344"/>
         <location filename="../src/PDFDocumentView.cpp" line="3354"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3361"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3366"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3364"/>
         <location filename="../src/PDFDocumentView.cpp" line="3371"/>
         <location filename="../src/PDFDocumentView.cpp" line="3376"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3377"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3378"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3379"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3380"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3381"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3386"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3387"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3388"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3389"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3390"/>
         <source>Denied</source>
         <translation>Refusé</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3350"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3360"/>
         <source>Assembling only</source>
         <translation>Uniquement assembler</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3351"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3361"/>
         <source>Insert, rotate, or delete pages and create bookmarks or thumbnail images</source>
         <translation>Insérer, faire tourner, ou supprimer des pages et créer des onglets ou de images signets</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3359"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3369"/>
         <source>Accessibility support only</source>
         <translation>Uniquement aide d&apos;accès</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3387"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3397"/>
         <source>Printing:</source>
         <translation>Impression :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3388"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3398"/>
         <source>Modifications:</source>
         <translation>Modifications :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3389"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3399"/>
         <source>Extraction:</source>
         <translation>Extraction :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3390"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3400"/>
         <source>Annotation:</source>
         <translation>Annotation :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3391"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3401"/>
         <source>Filling forms:</source>
         <translation>Formes de sauvegarde :</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3399"/>
-        <location filename="../src/PDFDocumentView.cpp" line="3491"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3409"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3501"/>
         <source>Annotations</source>
         <translation>Annotations</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3492"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3502"/>
         <source>Page</source>
         <translation>Page</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3492"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3502"/>
         <source>Subject</source>
         <translation>Sujet</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3492"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3502"/>
         <source>Author</source>
         <translation>Auteur</translation>
     </message>
     <message>
-        <location filename="../src/PDFDocumentView.cpp" line="3492"/>
+        <location filename="../src/PDFDocumentView.cpp" line="3502"/>
         <source>Contents</source>
         <translation>Table des matières</translation>
     </message>
