@@ -387,9 +387,6 @@ private:
 private:
   vector<string> formatsMade;
 
-private:
-  ofstream logStream;
-
 private:
   std::shared_ptr<MiKTeX::Packages::PackageManager> packageManager;
 
@@ -450,7 +447,6 @@ enum Option
   OPT_COMMON_DATA,              // <internal/>
   OPT_COMMON_INSTALL,           // <internal/>
   OPT_COMMON_ROOTS,             // <internal/>
-  OPT_LOG_FILE,                 // <internal/>
   OPT_DEFAULT_PAPER_SIZE,       // <internal/>
 #if defined(MIKTEX_WINDOWS)
   OPT_NO_REGISTRY,              // <internal/>
@@ -545,7 +541,7 @@ void IniTeXMFApp::Init(int argc, const char* argv[])
   LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString(TheNameOfTheGame, VersionNumber(MIKTEX_COMPONENT_VERSION_STR)));
   PathName cwd;
   cwd.SetToCurrentDirectory();
-  LOG4CXX_INFO(logger, fmt::format("this process ({0}) started by {1} in directory {2} with command line: {3}", thisProcess->GetSystemId(), Q_(invokerName), cwd.ToDisplayString(), CommandLineBuilder(argc, argv)));
+  LOG4CXX_INFO(logger, fmt::format("this process ({0}) started by {1} in directory {2} with command line: {3}", thisProcess->GetSystemId(), Q_(invokerName), cwd.ToDisplayString(), CommandLineBuilder(argc, argv).ToString()));
   FlushPendingTraceMessages();
   if (session->IsAdminMode())
   {
@@ -561,10 +557,6 @@ void IniTeXMFApp::Init(int argc, const char* argv[])
 
 void IniTeXMFApp::Finalize(bool keepSession)
 {
-  if (logStream.is_open())
-  {
-    logStream.close();
-  }
   FlushPendingTraceMessages();
   packageInstaller = nullptr;
   packageManager = nullptr;
@@ -931,7 +923,7 @@ void IniTeXMFApp::CreateConfigFile(const string& relPath, bool edit)
   }
   if (!File::Exists(configFile))
   {
-    Verbose(fmt::format(T_("Creating config file: {0}..."), configFile));
+    Verbose(fmt::format(T_("Creating config file: {0}..."), Q_(configFile.ToDisplayString())));
     if (!session->TryCreateFromTemplate(configFile))
     {
       File::WriteBytes(configFile, {});
@@ -1189,7 +1181,6 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
   vector<PathName> unregisterRoots;
   string defaultPaperSize;
   string engine;
-  string logFile;
   string portableRoot;
 
   bool optClean = false;
@@ -1328,11 +1319,6 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
       startupConfig.userConfigRoot = optArg;
       break;
 
-    case OPT_LOG_FILE:
-
-      logFile = optArg;
-      break;
-
     case OPT_MKLANGS:
 
       optMakeLanguageDat = true;
@@ -1486,12 +1472,6 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
     return;
   }
 
-  if (!logFile.empty())
-  {
-    auto mode = File::Exists(PathName(logFile)) ? ios_base::app : ios_base::out;
-    logStream = File::CreateOutputStream(PathName(logFile), mode);
-  }
-
   if (optPortable)
   {
     CreatePortableSetup(PathName(portableRoot));
